# 5.2 Gtekriterien fr Versuchsplne

# D-Effizienz des Lateinischen Quadrats LQ
LQ=matrix(c("0","0","0", "0","1","1", "0","2","2", "0","3","3",
            "1","0","3", "1","1","0", "1","2","1", "1","3","2",
            "2","0","2", "2","1","3", "2","2","0", "2","3","1", 
            "3","0","1", "3","1","2", "3","2","3", "3","3","0"),
            nrow=16,ncol=3,byrow=TRUE);
LQ=as.data.frame(LQ); names(LQ)=c("A","B","C")
X=as.matrix(model.matrix(~.,LQ)); J=dim(X)[1]
XsX=t(X)%*%X; XsXinv=solve(XsX); p=dim(X)[2]
D_LQ=1/(J*det(XsXinv)**(1/p)); D_LQ
# D-Effizienz des vollstndigen Versuchsplans VV zum Vergleich
VV=expand.grid(A=c("0","1","2","3"),B=c("0","1","2","3"),
               C=c("0","1","2","3"))
X=as.matrix(model.matrix(~.,VV)); J=dim(X)[1]
XsX=t(X)%*%X; XsXinv=solve(XsX); p=dim(X)[2]
D_VV=1/(J*det(XsXinv)**(1/p)); D_VV; D_LQ/D_VV

# 5.3 Konstruktion orthogonaler Versuchsplne

library(DoE.base) # Paket mit balancierten, orthogonalen Plnen
# Suche nach einem Plan, D-Effizienz
OA=oa.design(nlevels=c(4,4,4,4,4)) # Aufruf der Suche
design.info(OA)$generating.oa # Welcher Plan wurde genutzt?
VP=as.data.frame(as.matrix(OA)); VP # Ausgabe des Plans
X=as.matrix(model.matrix(~.,VP)); J=dim(X)[1]
XsX=t(X)%*%X; XsXinv=solve(XsX); p=dim(X)[2]
D_VP=1/(J*det(XsXinv)**(1/p)); D_VP # D-Effizienz
# D-Effizienz des vollstndigen Versuchsplans VV zum Vergleich
VV=expand.grid(A=c("0","1","2","3"),B=c("0","1","2","3"),
               C=c("0","1","2","3"),D=c("0","1","2","3"),
               E=c("0","1","2","3"))
X=as.matrix(model.matrix(~.,VV)); J=dim(X)[1]
XsX=t(X)%*%X; XsXinv=solve(XsX); p=dim(X)[2]
D_VV=1/(J*det(XsXinv)**(1/p)); D_VV; D_VP/D_VV

# 5.4 Konstruktion optimaler Versuchsplne

library(AlgDesign)
# Vollstndiger Versuchsplan mit Berechnung der D-Effizienz:
all=expand.grid(A=c("1","2"),B=c("1","2"),C=c("1","2"),
                D=c("1","2","3"),E=c("1","2","3"),
                F=c("1","2","3"),G=c("1","2","3"))
set.seed(1); n=dim(all)[1]
plan.all=optFederov(~.,data=all,nTrials=n); plan.all$D
# Reduzierter Versuchsplan nach Federov:
plan.red=optFederov(~.,data=all,nTrials=16,nRepeats=100)
plan.red$design; plan.red$D; plan.red$D/plan.all$D
# (zustzliche) "von Hand"-Berechnung der D-Effizienz:
X=as.matrix(model.matrix(~.,all)); n=dim(X)[1]
XsX=t(X)%*%X; XsXinv=solve(XsX); m=dim(X)[2]
plan.all.D=1/(n*det(XsXinv)**(1/m)); plan.all.D
X=as.matrix(model.matrix(~.,plan.red$design)); n=dim(X)[1]
XsX=t(X)%*%X; XsXinv=solve(XsX); m=dim(X)[2]
plan.red.D=1/(n*det(XsXinv)**(1/m)); plan.red.D
plan.red.D/plan.all.D

# 5.7.1 Versuchsplan fr eine traditionelle Conjointanalyse

library(AlgDesign) # 5 Merkmale mit 3 Ausprgungen
# Vollstndiger Versuchsplan mit Berechnung der D-Effizienz:
all=expand.grid(Hersteller=c("Grundig (A0)","LG (A1)","Samsung (A2)"),
      Diagonale=c("37 (94 cm, B0)","40 (101 cm, B1)","43 (108 cm, B2)"),
      Technologie=c("LCD (C0)","OLED (C1)","QLED (C2)"),
      Tuner=c("1 (D0)","2 (D1)","3 (D2)"),
      Preis=c("300 Euro (E0)","420 Euro (E1)","540 Euro (E2)"))
set.seed(1); n=dim(all)[1]
plan.all=optFederov(~.,data=all,nTrials=n); plan.all$D
# Reduzierter Versuchsplan nach Federov:
plan.red=optFederov(~.,data=all,nTrials=18,nRepeats=100)
plan.red$design; plan.red$D; plan.red$D/plan.all$D

# 5.7.2 Versuchsplan fr eine auswahlbasierte Conjointanalyse

library(DoE.base) # 6 Merkmale mit 3 Ausprgungen, eines mit 9
# Reduzierter Versuchsplan mit Berechnung der D-Effizienz:
OA=oa.design(nlevels=c(3,3,3,3,3,3,9),nruns=27) # Aufruf der Suche
design.info(OA)$generating.oa # Welcher Plan wurde genutzt?
VP=as.data.frame(as.matrix(OA)); VP$A=as.factor(VP$A)
VP$B=as.factor(VP$B); VP$C=as.factor(VP$C); VP$D=as.factor(VP$D)
VP$E=as.factor(VP$E)
levels(VP$A)[1]="Grundig (A0)"; levels(VP$A)[2]="LG (A1)"
levels(VP$A)[3]="Samsung (A2)"; levels(VP$B)[1]="37 (94 cm, B0)"
levels(VP$B)[2]="40 (101 cm, B1)"; levels(VP$B)[3]="43 (108 cm, B2)"
levels(VP$C)[1]="LCD (C0)"; levels(VP$C)[2]="OLED (C1)"
levels(VP$C)[3]="QLED (C2)"; levels(VP$D)[1]="1 (D0)"
levels(VP$D)[2]="2 (D1)"; levels(VP$D)[3]="3 (D2)"
levels(VP$E)[1]="300 Euro (E0)"
levels(VP$E)[2]="420 Euro (E1)"; levels(VP$E)[3]="540 Euro (E2)"
names(VP)=c("Hersteller","Diagonale","Technologie","Tuner","Preis","F","G")
VP=VP[order(VP$G,VP$F),]; VP # Ausgabe des Plans
X=as.matrix(model.matrix(~.,VP)); J=dim(X)[1]
XsX=t(X)%*%X; XsXinv=solve(XsX); p=dim(X)[2]
D_VP=1/(J*det(XsXinv)**(1/p)); D_VP # D-Effizienz
# D-Effizienz des vollstndigen Versuchsplans VV zum Vergleich
VV=expand.grid(Hersteller=c("Grundig (A0)","LG (A1)","Samsung (A2)"),
     Diagonale=c("37 (94 cm, B0)","40 (101 cm, B1)","43 (108 cm, B2)"),
     Technologie=c("LCD (C0)","OLED (C1)","QLED (C2)"),
     Tuner=c("1 (D0)","2 (D1)","3 (D2)"),
     Preis=c("300 Euro (E0)","420 E (E1)","540 Euro (E2)"),
     F=c("0","1","2"), G=c("0","1","2","3","4","5","6","7","8"))
X=as.matrix(model.matrix(~.,VV)); J=dim(X)[1]
XsX=t(X)%*%X; XsXinv=solve(XsX); p=dim(X)[2]
D_VV=1/(J*det(XsXinv)**(1/p)); D_VV; D_VP/D_VV
